/**************************************************************
 Water noize generator
	 for the PIC12F629
 **************************************************************
	author air_variable
	http://airvariable.asablo.jp/
	version 1.00
	Tools:
	MPLABX V1.95
	XC8 compiler v1.21
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20131211 22:00	1.00	---		Begin the program
---------------------------------------------------------------
port assignment
1	VDD					8	VSS
2	RA5	O LED			7	RA0	(PGD)
3	RA4	(not used)		6	RA1	(PGC)
4	RA3	I SWITCH(VPP)	5	RA2	O	PIEZO_OUT
 */

#include <xc.h>
#include <stdio.h>

// CONFIG
#pragma config FOSC = INTRCIO   // Oscillator Selection bits (INTOSC oscillator: I/O function on GP4/OSC2/CLKOUT pin, I/O function on GP5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-Up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = OFF      // GP3/MCLR pin function select (GP3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
#pragma config BOREN = OFF      // Brown-out Detect Enable bit (BOD disabled)
#pragma config CP = OFF         // Code Protection bit (Program Memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)

#define MHz 000000
#define _XTAL_FREQ 4MHz

//output port					//	  76543210
#define sound_output 	0x04	//GP2 00000100
#define led_output 		0x20 	//GP5 00100000
#define keyinput 		0x08 	//GP3 00001000
//output port (individual)
#define SoundPort	GP2
#define LEDPort		GP5


//Global settings
unsigned char ToneSet;
unsigned char TempoCount = 100; //Default TEMPO

//Interrupt service routine

void interrupt isr(void) {
	TMR0 = ToneSet;
	if (ToneSet != 0) {
		if (SoundPort) SoundPort = 0;
		else SoundPort = 1; //alternate port
	}
	T0IF = 0;
}

//tempo control


void sleep_control(void) {
	GIE = 0; //Int off
	T0IE = 0; //timer interrupt is off
	LEDPort = 0; //LED light
	__delay_ms(100);
	LEDPort = 1; //LED off
	while ((GPIO & keyinput) == 0); //wait if key is pushing
	__delay_ms(100);
	TRISIO = 0b11111111; //GPIO is all input
	NOP(); //Stabilize time
	WPU = 0b00110111; //ALL Pin Pull Up(then Low power)
	IOCB = keyinput; // checking switch port
	GPIF = 0; //GPIF flag clear
	GPIE = 1; //Sleep Wakeup enable
	GIE = 0; //Interrupt service is disable

	//vector non jump and execute next command
	SLEEP(); //sleep!
	NOP(); //Wake up

	// Wakeup when switch is LOW 
	GPIE = 0; //Sleep Wakeup disable
	GPIF = 0; //GPIF flag clear
	T0IE = 1; //timer on
	GPIO = GPIO; // Bug avoid
	TRISIO = ~(sound_output | led_output);
	GPIO &= ~led_output; //LED flash
	__delay_ms(100);
	while ((GPIO & keyinput) == 0); //wait if key is pushing
	__delay_ms(100);
	GIE = 1; //Interrupt is All enable
}

void delay_10ms(unsigned char temp) { //10ms multiple x times
	while (temp--) {
		__delay_ms(10);
	}
}

void main(void) {
	unsigned int n = 0;
	unsigned char tmp, tmp2;
	CMCON = 7; //All ports are digital I/O.
	OPTION_REG = 0b00000010; // prescale 8 Pullup enable
#if _12F675
	ANSEL = 0b00000000; //when device is 12F675
#endif
	TRISIO = ~(sound_output | led_output); //LED and piezo (GP2,GP5)
	GPIO = 0b00000000;
	TMR0 = 0x00;
	ToneSet = 187; //tone is A
	T0IE = 1;
	PEIE = 1;
	GIE = 1;
	LEDPort = 1;
	__delay_ms(200);
	LEDPort = 0;

	sleep_control(); //sleep
	while (1) {

		if (((GPIO & keyinput) == 0) || (n > (512 * 20))) {
			n = 0;
			sleep_control(); // resume
		}
		//ToneSet=tmp;
		ToneSet = ((unsigned char) (rand())% 226 ) ; //擾
		//wtime(tmp2);
		//wtime(1);
		__delay_ms(2);
		n++;
		if ((n % 512) > 20) {
			LEDPort = 1;
		} else {
			LEDPort = 0;
		}

	}
}
